function eom = computeEquationsOfMotion(q,qd,qdd,parts,joints,constraints,proj)
    if nargin == 1
        eom = q;
        q = eom.q;
        qd = eom.qd;
        qdd = eom.qdd;
        parts = eom.parts;
        joints = eom.joints;
        constraints = eom.constraints;
        proj = eom.proj;
    end

    partNames = cell([length(parts),1]);
    rootForces = {};
    for i = 1:length(parts)
        parts{i}.hasChild = false;
        cPName = parts{i}.name;
        partNames{i} = cPName;

        parts{i}.rootForceNames = { ['WForce_' cPName '_x'], ['WForce_' cPName '_y'], ['WForce_' cPName '_z'], ...
                                    ['WBTorque_' cPName '_x'], ['WBTorque_' cPName '_y'], ['WBTorque_' cPName '_z'], ...
                                    ['WTorque_' cPName '_x'], ['WTorque_' cPName '_y'], ['WTorque_' cPName '_z'] };
        rootForces = { rootForces{:}, parts{i}.rootForceNames{:} };
    end
    for i = 1:length(parts)
        if isfield(parts{i},'parentInfo')
            parentJ = find(strcmp(partNames,parts{i}.parentInfo.name),1);
            if ~isempty(parentJ)
                parts{parentJ}.hasChild = true;
            else
                error('Unable to find parent part: %s',parts{i}.parentInfo.name);
            end

            parts{i}.parentInfo.drelativeOrientdq = sparsesym(jacobian(parts{i}.parentInfo.relativeOrient,q));
            parts{i}.parentInfo.dparentJointPosdq = sparsesym(jacobian(parts{i}.parentInfo.parentJointPos,q));
            parts{i}.parentInfo.dchildJointPosdq = sparsesym(jacobian(parts{i}.parentInfo.childJointPos,q));

            parts{i}.parentInfo.d2relativeOrientd2q = jacobian(parts{i}.parentInfo.drelativeOrientdq,q);
            parts{i}.parentInfo.d2parentJointPosd2q = jacobian(parts{i}.parentInfo.dparentJointPosdq,q);
            parts{i}.parentInfo.d2childJointPosd2q = jacobian(parts{i}.parentInfo.dchildJointPosdq,q);
            
%             parts{i}.parentInfo.d3relativeOrientd3q = jacobian(parts{i}.parentInfo.d2relativeOrientd2q,q);
%             parts{i}.parentInfo.d3parentJointPosd3q = jacobian(parts{i}.parentInfo.d2parentJointPosd2q,q);
%             parts{i}.parentInfo.d3childJointPosd3q = jacobian(parts{i}.parentInfo.d2childJointPosd2q,q);
        else
            parts{i}.dCoMdq = sparsesym(jacobian(parts{i}.CoM,q));
            parts{i}.dOrientdq = sparsesym(jacobian(parts{i}.Orient,q));

            parts{i}.d2CoMd2q = jacobian(parts{i}.dCoMdq,q);
            parts{i}.d2Orientd2q = jacobian(parts{i}.dOrientdq,q);

%             parts{i}.d3CoMd3q = jacobian(parts{i}.d2CoMd2q,q);
%             parts{i}.d3Orientd3q = jacobian(parts{i}.d2Orientd2q,q);
        end
    end
    for i = 1:length(constraints)
        cconst = constraints{i};
        constraints{i} = struct;
        constraints{i}.eq = cconst;
        dconstdq = jacobian(cconst,q);
        constraints{i}.dconstdq = sparsesym(dconstdq);
        constraints{i}.d2constd2t = jacobian(dconstdq*qd,q)*qd;

%         constraints{i}.d2constd2q = jacobian(constraints{i}.dconstdq,q);
%         constraints{i}.d3constd3q = jacobian(constraints{i}.d2constd2q,q);
%         constraints{i}.d3constd2tdq = jacobian(constraints{i}.d2constd2t,q);
%         constraints{i}.d3constd2tdqd = jacobian(constraints{i}.d2constd2t,qd);
    end
    jointTorques = {};
    for i = 1:length(joints)
        cJName = joints{i}.name;
        parentJ = find(strcmp(partNames,joints{i}.parentName),1);
        if ~isempty(parentJ)
            joints{i}.parentInd = parentJ;
        else
            error('Unable to find parent part (%s) of joint %s', joints{i}.parentName, joints{i}.name);
        end
        
        childJ = find(strcmp(partNames,joints{i}.childName),1);
        if ~isempty(childJ)
            joints{i}.childInd = childJ;
        else
            error('Unable to find child part (%s) of joint %s', joints{i}.childName, joints{i}.name);
        end
        
        joints{i}.jointTorqueNames = { ['JTorque_' cJName '_x'], ['JTorque_' cJName '_y'], ['JTorque_' cJName '_z'] };
        jointTorques = { jointTorques{:}, joints{i}.jointTorqueNames{:} };     
    end
    eom.q = q;
    eom.qd = qd;
    eom.qdd = qdd;
    eom.parts = parts;
    eom.joints = joints;
    eom.constraints = constraints;
    eom.rootForces = rootForces;
    eom.jointTorques = jointTorques;
    if nargin > 6
        eom.proj = proj;
    end
end
