function eom = defineBodyModel(nTrunkParts)
    % geometry parameters
    syms Len_LowArm CoM_LowArm Radius_LowArm real;
    syms Len_UppArm CoM_UppArm Radius_UppArm real;
    syms Len_UppTrunk CoM_UppTrunk real;
    syms Len_MidTrunk CoM_MidTrunk real;
    syms Len_LowTrunk CoM_LowTrunk Width_Trunk_x Width_Trunk_y real;
    syms Len_Thigh CoM_Thigh Radius_Thigh real;
    syms Len_Shank CoM_Shank Radius_Shank real;
    syms Len_Foot CoM_Foot_x CoM_Foot_z Width_Foot Height_Foot real;
    syms LegSep Width_Shoulder Offset_Heel Offset_Shoulder_x Offset_Shoulder_z real;

    % mass parameters;
    syms Mass_LowArm positive
    syms Moment_LowArm_x Moment_LowArm_y Moment_LowArm_z positive;
    syms Mass_UppArm positive
    syms Moment_UppArm_x Moment_UppArm_y Moment_UppArm_z positive;
    syms Mass_UppTrunk positive
    syms Moment_UppTrunk_x Moment_UppTrunk_y Moment_UppTrunk_z positive;
    syms Mass_MidTrunk positive
    syms Moment_MidTrunk_x Moment_MidTrunk_y Moment_MidTrunk_z positive;
    syms Mass_LowTrunk positive
    syms Moment_LowTrunk_x Moment_LowTrunk_y Moment_LowTrunk_z positive;
    syms Mass_Thigh positive
    syms Moment_Thigh_x Moment_Thigh_y Moment_Thigh_z positive;
    syms Mass_Shank positive
    syms Moment_Shank_x Moment_Shank_y Moment_Shank_z positive;
    syms Mass_Foot positive
    syms Moment_Foot_x Moment_Foot_y Moment_Foot_z positive;

    % all parameters 
    params = [ ];
    params = [ params LegSep Width_Trunk_x Width_Trunk_y Width_Shoulder Offset_Shoulder_x Offset_Shoulder_z Radius_UppArm Radius_LowArm Offset_Heel ];
    params = [ params Len_LowArm CoM_LowArm Mass_LowArm Moment_LowArm_x Moment_LowArm_y Moment_LowArm_z ];
    params = [ params Len_UppArm CoM_UppArm Mass_UppArm Moment_UppArm_x Moment_UppArm_y Moment_UppArm_z ];
    params = [ params Len_UppTrunk CoM_UppTrunk Mass_UppTrunk Moment_UppTrunk_x Moment_UppTrunk_y Moment_UppTrunk_z ];
    params = [ params Len_MidTrunk CoM_MidTrunk Mass_MidTrunk Moment_MidTrunk_x Moment_MidTrunk_y Moment_MidTrunk_z ];
    params = [ params Len_LowTrunk CoM_LowTrunk Mass_LowTrunk Moment_LowTrunk_x Moment_LowTrunk_y Moment_LowTrunk_z ];
    params = [ params Len_Thigh Radius_Thigh CoM_Thigh Mass_Thigh Moment_Thigh_x Moment_Thigh_y Moment_Thigh_z ];
    params = [ params Len_Shank Radius_Shank CoM_Shank Mass_Shank Moment_Shank_x Moment_Shank_y Moment_Shank_z  ];
    params = [ params Len_Foot Width_Foot Height_Foot CoM_Foot_x CoM_Foot_z Mass_Foot Moment_Foot_x Moment_Foot_y Moment_Foot_z  ];

    % state variables
    % lower trunk position and orientation
    syms philtrw philtrdw philtrddw real;
    syms philtrx philtrdx philtrddx real;
    syms philtry philtrdy philtrddy real;
    syms philtrz philtrdz philtrddz real;
    syms rx rxd rxdd real;
    syms ry ryd rydd real;
    syms rz rzd rzdd real;

    % mid trunk orientation
    if nTrunkParts > 2
        syms psimtrw psimtrdw psimtrddw real;
        syms psimtrx psimtrdx psimtrddx real;
        syms psimtry psimtrdy psimtrddy real;
        syms psimtrz psimtrdz psimtrddz real;
    end

    % upper trunk orientation
    syms psiutrw psiutrdw psiutrddw real;
    syms psiutrx psiutrdx psiutrddx real;
    syms psiutry psiutrdy psiutrddy real;
    syms psiutrz psiutrdz psiutrddz real;

    % upper arm 1 orientation and derivatives
    syms psi1uaw psi1uadw psi1uaddw real;
    syms psi1uax psi1uadx psi1uaddx real;
    syms psi1uay psi1uady psi1uaddy real;
    syms psi1uaz psi1uadz psi1uaddz real;

    % lower arm 1 orientation and derivatives
    syms psi1law psi1ladw psi1laddw real;
    syms psi1lay psi1lady psi1laddy real;

    % thigh 1 orientation and derivatives
    syms psi1tw psi1tdw psi1tddw real;
    syms psi1tx psi1tdx psi1tddx real;
    syms psi1ty psi1tdy psi1tddy real;
    syms psi1tz psi1tdz psi1tddz real;
    % knee 1 joint angle and derivatives
    syms psi1kw psi1kdw psi1kddw real;
    syms psi1ky psi1kdy psi1kddy real;
    % ankle 1 joint angle and derivatives
    syms psi1aw psi1adw psi1addw real;
    syms psi1ax psi1adx psi1addx real;
    syms psi1ay psi1ady psi1addy real;

    % upper arm 2 orientation and derivatives
    syms psi2uaw psi2uadw psi2uaddw real;
    syms psi2uax psi2uadx psi2uaddx real;
    syms psi2uay psi2uady psi2uaddy real;
    syms psi2uaz psi2uadz psi2uaddz real;

    % lower arm 2 orientation and derivatives
    syms psi2law psi2ladw psi2laddw real;
    syms psi2lay psi2lady psi2laddy real;

    % thigh 2 orientation and derivatives
    syms psi2tw psi2tdw psi2tddw real;
    syms psi2tx psi2tdx psi2tddx real;
    syms psi2ty psi2tdy psi2tddy real;
    syms psi2tz psi2tdz psi2tddz real;
    % knee 2 joint angle and derivatives
    syms psi2kw psi2kdw psi2kddw real;
    syms psi2ky psi2kdy psi2kddy real;
    % ankle 2 joint angle and derivatives
    syms psi2aw psi2adw psi2addw real;
    syms psi2ax psi2adx psi2addx real;
    syms psi2ay psi2ady psi2addy real;

    constraints = {};
    if nTrunkParts > 1
        psiutr   = [   psiutrw;   psiutrx;   psiutry;   psiutrz ];
        psiutrd  = [  psiutrdw;  psiutrdx;  psiutrdy;  psiutrdz ];
        psiutrdd = [ psiutrddw; psiutrddx; psiutrddy; psiutrddz ];
        constraints = [ constraints { sum(psiutr.^2) - 1 } ];
    else
        psiutr   = [];
        psiutrd  = [];
        psiutrdd = [];
    end

    if nTrunkParts > 2
        psimtr   = [   psimtrw;   psimtrx;   psimtry;   psimtrz ];
        psimtrd  = [  psimtrdw;  psimtrdx;  psimtrdy;  psimtrdz ];
        psimtrdd = [ psimtrddw; psimtrddx; psimtrddy; psimtrddz ];
        constraints = [ constraints { sum(psimtr.^2) - 1 } ];
    else
        psimtr   = [];
        psimtrd  = [];
        psimtrdd = [];
    end

    philtr   = [   philtrw;   philtrx;   philtry;   philtrz ];
    philtrd  = [  philtrdw;  philtrdx;  philtrdy;  philtrdz ];
    philtrdd = [ philtrddw; philtrddx; philtrddy; philtrddz ];
    constraints = [ constraints { sum(philtr.^2) - 1 } ];

    root   = [   rx;   ry;   rz ];
    rootd  = [  rxd;  ryd;  rzd ];
    rootdd = [ rxdd; rydd; rzdd ];

    psi1ua   = [   psi1uaw;   psi1uax;   psi1uay;   psi1uaz ];
    psi1uad  = [  psi1uadw;  psi1uadx;  psi1uady;  psi1uadz ];
    psi1uadd = [ psi1uaddw; psi1uaddx; psi1uaddy; psi1uaddz ];
    constraints = [ constraints { sum(psi1ua.^2) - 1 } ];

    psi1la   = [   psi1law;   psi1lay ];
    psi1lad  = [  psi1ladw;  psi1lady ];
    psi1ladd = [ psi1laddw; psi1laddy ];
    constraints = [ constraints { sum(psi1la.^2) - 1 } ];

    psi1t   = [   psi1tw;   psi1tx;   psi1ty;   psi1tz ];
    psi1td  = [  psi1tdw;  psi1tdx;  psi1tdy;  psi1tdz ];
    psi1tdd = [ psi1tddw; psi1tddx; psi1tddy; psi1tddz ];
    constraints = [ constraints { sum(psi1t.^2) - 1 } ];

    psi1k   = [   psi1kw;   psi1ky ];
    psi1kd  = [  psi1kdw;  psi1kdy ];
    psi1kdd = [ psi1kddw; psi1kddy ];
    constraints = [ constraints { sum(psi1k.^2) - 1 } ];

    psi1a   = [   psi1aw;   psi1ax;   psi1ay ];
    psi1ad  = [  psi1adw;  psi1adx;  psi1ady ];
    psi1add = [ psi1addw; psi1addx; psi1addy ];
    constraints = [ constraints { sum(psi1a.^2) - 1 } ];

    psi2ua   = [   psi2uaw;   psi2uax;   psi2uay;   psi2uaz ];
    psi2uad  = [  psi2uadw;  psi2uadx;  psi2uady;  psi2uadz ];
    psi2uadd = [ psi2uaddw; psi2uaddx; psi2uaddy; psi2uaddz ];
    constraints = [ constraints { sum(psi2ua.^2) - 1 } ];

    psi2la   = [   psi2law;   psi2lay ];
    psi2lad  = [  psi2ladw;  psi2lady ];
    psi2ladd = [ psi2laddw; psi2laddy ];
    constraints = [ constraints { sum(psi2la.^2) - 1 } ];

    psi2t   = [   psi2tw;   psi2tx;   psi2ty;   psi2tz ];
    psi2td  = [  psi2tdw;  psi2tdx;  psi2tdy;  psi2tdz ];
    psi2tdd = [ psi2tddw; psi2tddx; psi2tddy; psi2tddz ];
    constraints = [ constraints { sum(psi2t.^2) - 1 } ];

    psi2k   = [   psi2kw;   psi2ky ];
    psi2kd  = [  psi2kdw;  psi2kdy ];
    psi2kdd = [ psi2kddw; psi2kddy ];
    constraints = [ constraints { sum(psi2k.^2) - 1 } ];

    psi2a   = [   psi2aw;   psi2ax;   psi2ay ];
    psi2ad  = [  psi2adw;  psi2adx;  psi2ady ];
    psi2add = [ psi2addw; psi2addx; psi2addy ];
    constraints = [ constraints { sum(psi2a.^2) - 1 } ];

    % generalized coordinate vectors
    q    = [   psi1t;   psi1k;   psi1a;   psi1ua;   psi1la;   psi2t;   psi2k;   psi2a;   psi2ua;   psi2la;   psiutr;   psimtr;   philtr;   root ];
    qd   = [  psi1td;  psi1kd;  psi1ad;  psi1uad;  psi1lad;  psi2td;  psi2kd;  psi2ad;  psi2uad;  psi2lad;  psiutrd;  psimtrd;  philtrd;  rootd ];
    qdd  = [ psi1tdd; psi1kdd; psi1add; psi1uadd; psi1ladd; psi2tdd; psi2kdd; psi2add; psi2uadd; psi2ladd; psiutrdd; psimtrdd; philtrdd; rootdd ];
    projq = [  projectQuat(psi1t); projectQuat(psi1k); projectQuat(psi1a); ...
               projectQuat(psi1ua); projectQuat(psi1la); ...
               projectQuat(psi2t); projectQuat(psi2k); projectQuat(psi2a); ...
               projectQuat(psi2ua); projectQuat(psi2la); ...
               projectQuat(psiutr); projectQuat(psimtr); projectQuat(philtr); root ];
    projqd = [ projectQuatd(psi1t,psi1td); projectQuatd(psi1k,psi1kd); projectQuatd(psi1a,psi1ad); ...
               projectQuatd(psi1ua,psi1uad); projectQuatd(psi1la,psi1lad); ...
               projectQuatd(psi2t,psi2td); projectQuatd(psi2k,psi2kd); projectQuatd(psi2a,psi2ad); ...
               projectQuatd(psi2ua,psi2uad); projectQuatd(psi2la,psi2lad); ...
               projectQuatd(psiutr,psiutrd); projectQuatd(psimtr,psimtrd); projectQuatd(philtr,philtrd); rootd; ];

    % Construct a set of body parts
    parts = {};

    if nTrunkParts == 1
        MidTrunk_RelCoM_LowTrunk = [0;0;-(Len_LowTrunk-CoM_LowTrunk+CoM_MidTrunk)];
        [Mc,Ic,CoMc] = combineParts(Mass_MidTrunk, diag([Moment_MidTrunk_x; Moment_MidTrunk_y; Moment_MidTrunk_z]),               zeros(3,1), eye(3), ...
                                    Mass_LowTrunk, diag([Moment_LowTrunk_x; Moment_LowTrunk_y; Moment_LowTrunk_z]), MidTrunk_RelCoM_LowTrunk, eye(3));
        MidTrunk_RelCoM_UppTrunk = [0;0; (Len_MidTrunk-CoM_MidTrunk+CoM_UppTrunk)];
        [Mc,Ic,CoMc] = combineParts(Mc, Ic, CoMc, eye(3), ...
                                    Mass_UppTrunk, diag([Moment_UppTrunk_x; Moment_UppTrunk_y; Moment_UppTrunk_z]), MidTrunk_RelCoM_UppTrunk, eye(3));

        Trunk.name = 'Trunk';
        Trunk.CoM = root;
        Trunk.Orient = philtr;
        Trunk.geometryType = 'cylZ';
        Trunk.geometryParams = [ 0.5*Width_Trunk_x; 0.5*Width_Trunk_y; (Len_LowTrunk+Len_MidTrunk+Len_UppTrunk); CoMc ];
        Trunk.RelCoordTransform = [ diag([0.5*Width_Trunk_x; 0.5*Width_Trunk_y; 0.5*(Len_LowTrunk+Len_MidTrunk+Len_UppTrunk)]) [0;0;0.5*(Len_LowTrunk+Len_MidTrunk+Len_UppTrunk)] - CoMc ];
        Trunk.mass = Mc;
        Trunk.inertia = Ic;

        parts = [parts {Trunk}];

        thighParentPart = 'Trunk';
        armParentPart = 'Trunk';
    elseif nTrunkParts == 2
        LowTrunk.name = 'LowTrunk';
        LowTrunk.CoM = root;
        LowTrunk.Orient = philtr;
        LowTrunk.geometryType = 'cylZ';
        LowTrunk.geometryParams = [ 0.5*Width_Trunk_x; 0.5*Width_Trunk_y; Len_LowTrunk; 0; 0; CoM_LowTrunk ];
        LowTrunk.RelCoordTransform = [ diag([0.5*Width_Trunk_x; 0.5*Width_Trunk_y; 0.5*Len_LowTrunk]) [0;0;0.5*Len_LowTrunk-CoM_LowTrunk] ];
        LowTrunk.mass = Mass_LowTrunk;
        LowTrunk.inertia = diag([Moment_LowTrunk_x; Moment_LowTrunk_y; Moment_LowTrunk_z]);
        parts = [parts {LowTrunk}];
        
        MidTrunk_RelCoM_UppTrunk = [0;0; (Len_MidTrunk-CoM_MidTrunk+CoM_UppTrunk)];
        [Mc,Ic,CoMc] = combineParts(Mass_MidTrunk, diag([Moment_MidTrunk_x; Moment_MidTrunk_y; Moment_MidTrunk_z]),       [0;0;CoM_MidTrunk], eye(3), ...
                                    Mass_UppTrunk, diag([Moment_UppTrunk_x; Moment_UppTrunk_y; Moment_UppTrunk_z]), MidTrunk_RelCoM_UppTrunk, eye(3));

        UppTrunk.name = 'UppTrunk';
        parentJointPos = [0;0;Len_LowTrunk-CoM_LowTrunk];
        childJointPos = CoMc;
        UppTrunk.Orient = psiutr;
        UppTrunk.CoM = LowTrunk.CoM + quatrot(LowTrunk.Orient,parentJointPos) + quatrot(UppTrunk.Orient,childJointPos);
        UppTrunk.geometryType = 'cylZ';
        UppTrunk.geometryParams = [ 0.5*Width_Trunk_x; 0.5*Width_Trunk_y; Len_UppTrunk+Len_MidTrunk; childJointPos ];
        UppTrunk.RelCoordTransform = [ diag([0.5*Width_Trunk_x; 0.5*Width_Trunk_y; 0.5*(Len_UppTrunk+Len_MidTrunk)]) [0;0;0.5*(Len_UppTrunk+Len_MidTrunk)] - childJointPos ];
        UppTrunk.mass = Mc;
        UppTrunk.inertia = Ic;
        parts = [parts {UppTrunk}];

        thighParentPart = 'LowTrunk';
        armParentPart = 'UppTrunk';
    else
        LowTrunk.name = 'LowTrunk';
        LowTrunk.CoM = root;
        LowTrunk.Orient = philtr;
        LowTrunk.geometryType = 'cylZ';
        LowTrunk.geometryParams = [ 0.5*Width_Trunk_x; 0.5*Width_Trunk_y; Len_LowTrunk; 0; 0; CoM_LowTrunk ];
        LowTrunk.RelCoordTransform = [ diag([0.5*Width_Trunk_x; 0.5*Width_Trunk_y; 0.5*Len_LowTrunk]) [0;0;0.5*Len_LowTrunk-CoM_LowTrunk] ];
        LowTrunk.mass = Mass_LowTrunk;
        LowTrunk.inertia = diag([Moment_LowTrunk_x; Moment_LowTrunk_y; Moment_LowTrunk_z]);
        parts = [parts {LowTrunk}];

        MidTrunk.name = 'MidTrunk';
        parentJointPos = [0;0;Len_LowTrunk-CoM_LowTrunk];
        childJointPos = [0;0;CoM_MidTrunk];
        MidTrunk.Orient = psimtr;
        MidTrunk.CoM = LowTrunk.CoM + quatrot(LowTrunk.Orient,parentJointPos) + quatrot(MidTrunk.Orient,childJointPos);

        MidTrunk.geometryType = 'cylZ';
        MidTrunk.geometryParams = [ 0.5*Width_Trunk_x; 0.5*Width_Trunk_y; Len_MidTrunk; 0; 0; CoM_MidTrunk ];
        MidTrunk.RelCoordTransform = [ diag([0.5*Width_Trunk_x; 0.5*Width_Trunk_y; 0.5*Len_MidTrunk]) [0;0;0.5*Len_MidTrunk-CoM_MidTrunk] ];
        MidTrunk.mass = Mass_MidTrunk;
        MidTrunk.inertia = diag([Moment_MidTrunk_x; Moment_MidTrunk_y; Moment_MidTrunk_z]);
        parts = [parts {MidTrunk}];

        UppTrunk.name = 'UppTrunk';
        parentJointPos = [0;0;Len_MidTrunk-CoM_MidTrunk];
        childJointPos = [0;0;CoM_UppTrunk];
        UppTrunk.Orient = psiutr;
        UppTrunk.CoM = MidTrunk.CoM + quatrot(MidTrunk.Orient,parentJointPos) + quatrot(UppTrunk.Orient,childJointPos);
        UppTrunk.geometryType = 'cylZ';
        UppTrunk.geometryParams = [ 0.5*Width_Trunk_x; 0.5*Width_Trunk_y; Len_UppTrunk; 0; 0; CoM_UppTrunk ];
        UppTrunk.RelCoordTransform = [ diag([0.5*Width_Trunk_x; 0.5*Width_Trunk_y; 0.5*Len_UppTrunk]) [0;0;0.5*Len_UppTrunk-CoM_UppTrunk] ];
        UppTrunk.mass = Mass_UppTrunk;
        UppTrunk.inertia = diag([Moment_UppTrunk_x; Moment_UppTrunk_y; Moment_UppTrunk_z]);
        parts = [parts {UppTrunk}];

        thighParentPart = 'LowTrunk';
        armParentPart = 'UppTrunk';
    end
    
    LeftUppArm.name = 'LeftUppArm';
    LeftUppArm.parentInfo.name = armParentPart;
    LeftUppArm.parentInfo.relativeOrient = psi1ua;
    LeftUppArm.parentInfo.parentJointPos = [Offset_Shoulder_x;0.5*Width_Shoulder;Len_UppTrunk-CoM_UppTrunk+Offset_Shoulder_z];
    LeftUppArm.parentInfo.childJointPos = [0;0;-CoM_UppArm];
    LeftUppArm.geometryType = 'cylZ';
    LeftUppArm.geometryParams = [ Radius_UppArm; Radius_UppArm; Len_UppArm; 0; 0; Len_UppArm-CoM_UppArm ];
    LeftUppArm.RelCoordTransform = [ diag([Radius_UppArm; Radius_UppArm; 0.5*Len_UppArm]) [0;0;CoM_UppArm-0.5*Len_UppArm] ];
    LeftUppArm.mass = Mass_UppArm;
    LeftUppArm.inertia = diag([Moment_UppArm_x; Moment_UppArm_y; Moment_UppArm_z]);
    parts = [parts {LeftUppArm}];

    LeftLowArm.name = 'LeftLowArm';
    LeftLowArm.parentInfo.name = 'LeftUppArm';
    LeftLowArm.parentInfo.relativeOrient = [ psi1law; 0; psi1lay; 0 ];
    LeftLowArm.parentInfo.parentJointPos = [0;0;CoM_UppArm-Len_UppArm];
    LeftLowArm.parentInfo.childJointPos = [0;0;-CoM_LowArm];
    LeftLowArm.geometryType = 'cylZ';
    LeftLowArm.geometryParams = [ Radius_LowArm; Radius_LowArm; Len_LowArm; 0; 0; Len_LowArm-CoM_LowArm ];
    LeftLowArm.RelCoordTransform = [ diag([Radius_LowArm; Radius_LowArm; 0.5*Len_LowArm]) [0;0;CoM_LowArm-0.5*Len_LowArm] ];
    LeftLowArm.mass = Mass_LowArm;
    LeftLowArm.inertia = diag([Moment_LowArm_x; Moment_LowArm_y; Moment_LowArm_z]);
    parts = [parts {LeftLowArm}];

    RightUppArm.name = 'RightUppArm';
    RightUppArm.parentInfo.name = armParentPart;
    RightUppArm.parentInfo.relativeOrient = psi2ua;
    RightUppArm.parentInfo.parentJointPos = [Offset_Shoulder_x;-0.5*Width_Shoulder;Len_UppTrunk-CoM_UppTrunk+Offset_Shoulder_z];
    RightUppArm.parentInfo.childJointPos = [0;0;-CoM_UppArm];
    RightUppArm.geometryType = 'cylZ';
    RightUppArm.geometryParams = [ Radius_UppArm; Radius_UppArm; Len_UppArm; 0; 0; Len_UppArm-CoM_UppArm ];
    RightUppArm.RelCoordTransform = [ diag([Radius_UppArm; Radius_UppArm; 0.5*Len_UppArm]) [0;0;CoM_UppArm-0.5*Len_UppArm] ];
    RightUppArm.mass = Mass_UppArm;
    RightUppArm.inertia = diag([Moment_UppArm_x; Moment_UppArm_y; Moment_UppArm_z]);
    parts = [parts {RightUppArm}];

    RightLowArm.name = 'RightLowArm';
    RightLowArm.parentInfo.name = 'RightUppArm';
    RightLowArm.parentInfo.relativeOrient = [ psi2law; 0; psi2lay; 0 ];
    RightLowArm.parentInfo.parentJointPos = [0;0;CoM_UppArm-Len_UppArm];
    RightLowArm.parentInfo.childJointPos = [0;0;-CoM_LowArm];
    RightLowArm.geometryType = 'cylZ';
    RightLowArm.geometryParams = [ Radius_LowArm; Radius_LowArm; Len_LowArm; 0; 0; Len_LowArm-CoM_LowArm ];
    RightLowArm.RelCoordTransform = [ diag([Radius_LowArm; Radius_LowArm; 0.5*Len_LowArm]) [0;0;CoM_LowArm-0.5*Len_LowArm] ];
    RightLowArm.mass = Mass_LowArm;
    RightLowArm.inertia = diag([Moment_LowArm_x; Moment_LowArm_y; Moment_LowArm_z]);
    parts = [parts {RightLowArm}];

    LeftThigh.name = 'LeftThigh';
    LeftThigh.parentInfo.name = thighParentPart;
    LeftThigh.parentInfo.relativeOrient = psi1t;
    LeftThigh.parentInfo.parentJointPos = [0;Radius_Thigh+LegSep;-CoM_LowTrunk];
    LeftThigh.parentInfo.childJointPos = [0;0;-CoM_Thigh];
    LeftThigh.geometryType = 'cylZ';
    LeftThigh.geometryParams = [ Radius_Thigh; Radius_Thigh; Len_Thigh; 0; 0; Len_Thigh-CoM_Thigh ];
    LeftThigh.RelCoordTransform = [ diag([Radius_Thigh; Radius_Thigh; 0.5*Len_Thigh]) [0;0;CoM_Thigh-0.5*Len_Thigh] ];
    LeftThigh.mass = Mass_Thigh;
    LeftThigh.inertia = diag([Moment_Thigh_x; Moment_Thigh_y; Moment_Thigh_z]);
    parts = [parts {LeftThigh}];

    LeftShank.name = 'LeftShank';
    LeftShank.parentInfo.name = 'LeftThigh';
    LeftShank.parentInfo.relativeOrient = [ psi1kw; 0; psi1ky; 0 ];
    LeftShank.parentInfo.parentJointPos = [0;0;CoM_Thigh-Len_Thigh];
    LeftShank.parentInfo.childJointPos = [0;0;-CoM_Shank];
    LeftShank.geometryType = 'cylZ';
    LeftShank.geometryParams = [ Radius_Shank; Radius_Shank; Len_Shank; 0; 0; Len_Shank-CoM_Shank ];
    LeftShank.RelCoordTransform = [ diag([Radius_Shank; Radius_Shank; 0.5*Len_Shank]) [0;0;CoM_Shank-0.5*Len_Shank] ];
    LeftShank.mass = Mass_Shank;
    LeftShank.inertia = diag([Moment_Shank_x; Moment_Shank_y; Moment_Shank_z]);
    parts = [parts {LeftShank}];

    LeftFoot.name = 'LeftFoot';
    LeftFoot.geometryType = 'foot';
    LeftFoot.geometryParams = [ Len_Foot+Radius_Shank; Width_Foot; 0 ];
    LeftFoot.parentInfo.name = 'LeftShank';
    LeftFoot.parentInfo.relativeOrient = [psi1a; 0];
    LeftFoot.parentInfo.parentJointPos = [0;0;CoM_Shank-Len_Shank];
    LeftFoot.parentInfo.childJointPos = [CoM_Foot_x-Offset_Heel;0;-CoM_Foot_z];
    LeftFoot.RelCoordTransform = [ diag([0.5*Len_Foot; 0.5*Width_Foot; 0.5*Height_Foot]) [0.5*Len_Foot - CoM_Foot_x; 0; CoM_Foot_z - 0.5*Height_Foot] ];
    LeftFoot.mass = Mass_Foot;
    LeftFoot.inertia = diag([Moment_Foot_x; Moment_Foot_y; Moment_Foot_z]);
    parts = [parts {LeftFoot}];

    RightThigh.name = 'RightThigh';
    RightThigh.parentInfo.name = thighParentPart;
    RightThigh.parentInfo.relativeOrient = psi2t;
    RightThigh.parentInfo.parentJointPos = [0;-(Radius_Thigh+LegSep);-CoM_LowTrunk];
    RightThigh.parentInfo.childJointPos = [0;0;-CoM_Thigh];
    RightThigh.geometryType = 'cylZ';
    RightThigh.geometryParams = [ Radius_Thigh; Radius_Thigh; Len_Thigh; 0; 0; Len_Thigh-CoM_Thigh ];
    RightThigh.RelCoordTransform = [ diag([Radius_Thigh; Radius_Thigh; 0.5*Len_Thigh]) [0;0;CoM_Thigh-0.5*Len_Thigh] ];
    RightThigh.mass = Mass_Thigh;
    RightThigh.inertia = diag([Moment_Thigh_x; Moment_Thigh_y; Moment_Thigh_z]);
    parts = [parts {RightThigh}];

    RightShank.name = 'RightShank';
    RightShank.parentInfo.name = 'RightThigh';
    RightShank.parentInfo.relativeOrient = [ psi2kw; 0; psi2ky; 0 ];
    RightShank.parentInfo.parentJointPos = [0;0;CoM_Thigh-Len_Thigh];
    RightShank.parentInfo.childJointPos = [0;0;-CoM_Shank];
    RightShank.geometryType = 'cylZ';
    RightShank.geometryParams = [ Radius_Shank; Radius_Shank; Len_Shank; 0; 0; Len_Shank-CoM_Shank ];
    RightShank.RelCoordTransform = [ diag([Radius_Shank; Radius_Shank; 0.5*Len_Shank]) [0;0;CoM_Shank-0.5*Len_Shank] ];
    RightShank.mass = Mass_Shank;
    RightShank.inertia = diag([Moment_Shank_x; Moment_Shank_y; Moment_Shank_z]);
    parts = [parts {RightShank}];

    RightFoot.name = 'RightFoot';
    RightFoot.parentInfo.name = 'RightShank';
    RightFoot.parentInfo.relativeOrient = [psi2a; 0];
    RightFoot.parentInfo.parentJointPos = [0;0;CoM_Shank-Len_Shank];
    RightFoot.parentInfo.childJointPos = [CoM_Foot_x-Offset_Heel;0;-CoM_Foot_z];
    RightFoot.geometryType = 'foot';
    RightFoot.geometryParams = [ Len_Foot+Radius_Shank; Width_Foot; 0 ];
    RightFoot.RelCoordTransform = [ diag([0.5*Len_Foot; 0.5*Width_Foot; 0.5*Height_Foot]) [0.5*Len_Foot - CoM_Foot_x; 0; CoM_Foot_z - 0.5*Height_Foot] ];
    RightFoot.mass = Mass_Foot;
    RightFoot.inertia = diag([Moment_Foot_x; Moment_Foot_y; Moment_Foot_z]);
    parts = [parts {RightFoot}];

    % joints
    joints = {};

    LeftShoulderJoint.name = 'LeftShoulder';
    LeftShoulderJoint.parentName = armParentPart;
    LeftShoulderJoint.childName = 'LeftUppArm';
    LeftShoulderJoint.torques = true(3,1);
    joints = [joints {LeftShoulderJoint}];

    LeftElbowJoint.name = 'LeftElbow';
    LeftElbowJoint.parentName = 'LeftUppArm';
    LeftElbowJoint.childName = 'LeftLowArm';
    LeftElbowJoint.torques = [0;1;0] == 1;
    joints = [joints {LeftElbowJoint}];

    LeftHipJoint.name = 'LeftHip';
    LeftHipJoint.parentName = thighParentPart;
    LeftHipJoint.childName = 'LeftThigh';
    LeftHipJoint.torques = true(3,1);
    joints = [joints {LeftHipJoint}];

    LeftKneeJoint.name = 'LeftKnee';
    LeftKneeJoint.parentName = 'LeftThigh';
    LeftKneeJoint.childName = 'LeftShank';
    LeftKneeJoint.torques = [0;1;0] == 1;
    joints = [joints {LeftKneeJoint}];

    LeftAnkleJoint.name = 'LeftAnkle';
    LeftAnkleJoint.parentName = 'LeftShank';
    LeftAnkleJoint.childName = 'LeftFoot';
    LeftAnkleJoint.torques = [1;1;0] == 1;
    joints = [joints {LeftAnkleJoint}];
 
    RightShoulderJoint.name = 'RightShoulder';
    RightShoulderJoint.parentName = armParentPart;
    RightShoulderJoint.childName = 'RightUppArm';
    RightShoulderJoint.torques = true(3,1);
    joints = [joints {RightShoulderJoint}];

    RightElbowJoint.name = 'RightElbow';
    RightElbowJoint.parentName = 'RightUppArm';
    RightElbowJoint.childName = 'RightLowArm';
    RightElbowJoint.torques = [0;1;0] == 1;
    joints = [joints {RightElbowJoint}];

    RightHipJoint.name = 'RightHip';
    RightHipJoint.parentName = thighParentPart;
    RightHipJoint.childName = 'RightThigh';
    RightHipJoint.torques = true(3,1);
    joints = [joints {RightHipJoint}];

    RightKneeJoint.name = 'RightKnee';
    RightKneeJoint.parentName = 'RightThigh';
    RightKneeJoint.childName = 'RightShank';
    RightKneeJoint.torques = [0;1;0] == 1;
    joints = [joints {RightKneeJoint}];

    RightAnkleJoint.name = 'RightAnkle';
    RightAnkleJoint.parentName = 'RightShank';
    RightAnkleJoint.childName = 'RightFoot';
    RightAnkleJoint.torques = [1;1;0] == 1;
    joints = [joints {RightAnkleJoint}];

    if nTrunkParts == 3
        umTrunkJoint.name = 'UppMidTrunk';
        umTrunkJoint.parentName = 'UppTrunk';
        umTrunkJoint.childName = 'MidTrunk';
        umTrunkJoint.torques = true(3,1);
        joints = [joints {umTrunkJoint}];

        mlTrunkJoint.name = 'MidLowTrunk';
        mlTrunkJoint.parentName = 'MidTrunk';
        mlTrunkJoint.childName = 'LowTrunk';
        mlTrunkJoint.torques = true(3,1);
        joints = [joints {mlTrunkJoint}];
    elseif nTrunkParts == 2
        ulTrunkJoint.name = 'UppLowTrunk';
        ulTrunkJoint.parentName = 'UppTrunk';
        ulTrunkJoint.childName = 'LowTrunk';
        ulTrunkJoint.torques = true(3,1);
        joints = [joints {ulTrunkJoint}];
    end

    proj = [projq;projqd];
    
    eom.q = q;
    eom.qd = qd;
    eom.qdd = qdd;
    eom.parts = parts;
    eom.joints = joints;
    eom.constraints = constraints;
    eom.proj = proj;

    eom.params = params;
    eom.name = sprintf('FullBody_%dPartTrunk',nTrunkParts);
end

function q = projectQuat(q)
    if ~isempty(q)
        q = q/norm(q);
    end
end

function qd = projectQuatd(q,qd)
    if ~isempty(q)
        qd = qd - (dot(q,qd)/sum(q.^2))*q;
    end
end

function n = norm(q)
    n = sqrt(sum(q.^2));
end

