function deriveEquations(outPath)
    currPath = pwd;
    cd(outPath);

    %% Construct the model
    model = defineBodyModel(2);
    eom = computeEquationsOfMotion(model);

    %% Output the equations of motion
    writeTMTEquations(sprintf('eom%sEquations',eom.name),eom);
    writeProjFunc(sprintf('eom%sProj',eom.name),eom);

    %% Output the base optimization structure
    par = getBaseParameters;

    % phito = quataxisangle([0;1;0], pi/4);
    psiutr = [1;0;0;0];
    omegautr = [0;0;0];
    psiutrd = 0.5*quatmult([0;omegautr],psiutr);

    psimtr = [];
%     omegamtr = [];
%     psimtrd = 0.5*quatmult([0;omegamtr],psimtr);
    psimtrd = [];

    philtr = quataxisangle([0;0;1], pi/2);
    omegaltr = [0;0;0];
    philtrd = 0.5*quatmult([0;omegaltr],philtr);

    root  = [0;0;par.CoM_LowTrunk+par.Len_Thigh+par.Len_Shank+par.Height_Foot];
    rootd  = [0;0;0];

    psi1t = quataxisangle([-1;1;0]/sqrt(2), 0);
    omega1t = [0;0;0];
    psi1td = 0.5*quatmult([0;omega1t],psi1t);

    psi1k = [cos(pi/4);sin(pi/4)];
    psi1kd = [0;0];

    psi1a = [1;0;0];
    psi1ad = [0;0;0];

    psi1ua = [1;0;0;0];
    psi1uad = [0;0;0;0];

    psi1la = [1;0;];
    psi1lad = [0;0];

    psi2t = quataxisangle([-1;1;0]/sqrt(2), 0);
    omega2t = [0;0;0];
    psi2td = 0.5*quatmult([0;omega2t],psi2t);

    psi2k = [cos(pi/8);sin(pi/8)];
    psi2kd = [0;0];

    psi2a = [1;0;0];
    psi2ad = [0;0;0];

    psi2ua = [1;0;0;0];
    psi2uad = [0;0;0;0];

    psi2la = [1;0];
    psi2lad = [0;0];

    q0    = [   psi1t;   psi1k;   psi1a;   psi1ua;   psi1la;   psi2t;   psi2k;   psi2a;   psi2ua;   psi2la;   psiutr;   psimtr;   philtr;   root ];
    qd0   = [  psi1td;  psi1kd;  psi1ad;  psi1uad;  psi1lad;  psi2td;  psi2kd;  psi2ad;  psi2uad;  psi2lad;  psiutrd;  psimtrd;  philtrd;  rootd ];

    opts = struct;
    opts.par = par;
    opts.rootForces = eom.rootForces;
    opts.jointTorques = eom.jointTorques;

    opts.effectiveJointTorques = [];
    for i = 1:length(eom.joints)
        opts.effectiveJointTorques = [opts.effectiveJointTorques; eom.joints{i}.torques];
    end
    opts.effectiveJointTorques = opts.effectiveJointTorques == 1;
    
    opts.qNames = cell(length(eom.q),1);
    for i = 1:length(eom.q)
        opts.qNames{i} = char(eom.q(i));
    end

    opts.qdNames = cell(length(eom.qd),1);
    for i = 1:length(eom.q)
        opts.qdNames{i} = char(eom.qd(i));
    end
    
    opts.xNames = {opts.qNames{:} opts.qdNames{:}};

    for i = 1:length(eom.parts)
        opts.partNames{i} = eom.parts{i}.name;
    end
    
    for i = 1:length(eom.joints)
        opts.jointNames{i} = eom.joints{i}.name;
    end

    opts.xDims = length(opts.xNames);
    opts.qDims = length(opts.qNames);
    opts.nParts = length(opts.partNames);
    opts.nJoints = length(opts.jointNames);
    opts.fDims = 9*opts.nParts;
    opts.tauDims = 3*opts.nJoints;
    opts.eTauDims = sum(opts.effectiveJointTorques);

    opts.equationsFunc = str2func(sprintf('eom%sEquations',eom.name));
    opts.projFunc = str2func(sprintf('eom%sProj',eom.name));

    opts.q0 = q0;
    opts.qd0 = qd0;

    opts.modelName = sprintf('%sModel',eom.name);

    save(sprintf('eom%sOptBase.mat',eom.name),'opts');
    cd(currPath);

% end
