function writeTMTEquations(funcname,eom)
    sparseThresh = 0.05;
    
    q = eom.q;
    parts = eom.parts;
    joints = eom.joints;
    consts = eom.constraints;
    
    nParts = length(parts);
    nJoints = length(joints);

    qDims = length(q);
    kDims = 7*nParts;

    fid = fopen([funcname '.m'],'wt');
    
    fprintf(fid,'function eqs = %s(x,par,partInfo)\n',funcname);
    
    fprintf(fid,'\tif nargin < 3, partInfo = true; end\n');

    writeTMTHeaderInfo(fid,eom);

    fprintf(fid,'\n\t%% compute info for each part\n');
    for i = 1:nParts
        writeTMTPartInfo(fid,parts{i},qDims,sparseThresh,true,true,true,2);
    end
    
    fprintf(fid,'\n\t%% compute info for each joint\n');
    for i = 1:nJoints
        writeTMTJointInfo(fid,joints{i},qDims,kDims)
    end

    writeTMTMatrices(fid,eom,sparseThresh,1,1,1,1,1,1);

    fprintf(fid,'\n\t%% Put together the returning strutures\n');
    fprintf(fid,'\teqs = struct(''T'',T,''M'',M,''C'',C,''F'',F,''A'',A,''D'',D,''c'',c);\n');
    fprintf(fid,'\tif partInfo\n');
    for i = 1:nParts
        cPart = parts{i};
        fprintf(fid,'\t\teqs.parts{%d}.name = ''%s'';\n',i,cPart.name);

        printSymMatrix(fid,sprintf('eqs.parts{%d}.mass',i),cPart.mass,false,sparseThresh,2);
        fprintf(fid,'\t\teqs.parts{%d}.inertia = j%s;\n',i,cPart.name);

        fprintf(fid,'\t\teqs.parts{%d}.CoM = %sCoM;\n',i,cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.dCoMdq = d%sCoMdq;\n',i,cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.d2CoMd2q = d2%sCoMd2q;\n',i,cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.dCoMdt = d%sCoMdq*qd;\n',i,cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.d2CoMdtdq = d2CoM%sdtdq;\n',i,cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.d2CoMdtdqd = d2CoM%sdtdqd;\n',i,cPart.name);

        fprintf(fid,'\t\teqs.parts{%d}.Orient = %sOrient;\n',i,cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.dOrientdq = d%sOrientdq;\n',i,cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.d2Orientd2q = d2%sOrientd2q;\n',i,cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.dOrientdt = d%sOrientdq*qd;\n',i,cPart.name);

        fprintf(fid,'\t\teqs.parts{%d}.R = R%s;\n',i,cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.dRdq = quatrotderiv(%sOrient)*d%sOrientdq;\n', i, cPart.name, cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.dRdt = reshape(quatrotderiv(%sOrient)*d%sOrientdq*qd,[3,3]);\n', i, cPart.name, cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.d2Rdtdq = d2R%sdtdq;\n',i,cPart.name);
        fprintf(fid,'\t\teqs.parts{%d}.d2Rdtdqd = d2R%sdtdqd;\n',i,cPart.name);

        fprintf(fid,'\t\teqs.parts{%d}.geomType = ''%s'';\n',i,cPart.geometryType);
        printSymMatrix(fid,sprintf('eqs.parts{%d}.geomParams',i),cPart.geometryParams,false,sparseThresh,2);
    end
    fprintf(fid,'\tend\n');
    
    fclose(fid);
end
