function writeTMTMatrices(fid,eom,sparseThresh,T,M,A,C,F,Consts)
    parts = eom.parts;
    joints = eom.joints;
    consts = eom.constraints;
    
    nParts = length(parts);
    nJoints = length(joints);
    nConsts = length(consts);

    qDims = length(eom.q);
    kDims = 7*nParts;

    if T
        fprintf(fid,'\n\t%% setup T\n');
        fprintf(fid,'\tT = [\n');
        for i = 1:nParts
            fprintf(fid,'\t\td%sCoMdq; d%sOrientdq; \n',parts{i}.name,parts{i}.name);
        end
        fprintf(fid,'\t];\n');
    end

    if M
        fprintf(fid,'\n\t%% setup M\n');
        fprintf(fid,'\tM = zeros([%d,%d]);\n',kDims,kDims);
        for i = 1:nParts
            fprintf(fid,'\tM(%d+(1:3),%d+(1:3)) = M%s;\n',7*(i-1),7*(i-1),parts{i}.name);
            fprintf(fid,'\tM(%d+(4:7),%d+(4:7)) = I%s;\n',7*(i-1),7*(i-1),parts{i}.name);
        end
    end

    if A || T
        fprintf(fid,'\n\t%% setup S\n');
        fprintf(fid,'\tS = [\n');
        for i = 1:nParts
            fprintf(fid,'\t\td2%sCoMd2q; d2%sOrientd2q;\n',parts{i}.name,parts{i}.name);
        end
        fprintf(fid,'\t];\n');
    end

    if A
        fprintf(fid,'\n\t%% compute A\n');
        fprintf(fid,'\tqdOutProd = reshape(qd*qd.'',[%d,1]);\n', qDims^2);
        fprintf(fid,'\tG = S * qdOutProd;\n');
        fprintf(fid,'\tA = [\n');
        for i = 1:nParts
            fprintf(fid,'\t\tM%s*G((%d*7)+(1:3)); fOrient%s - I%s*G((%d*7)+(4:7));\n',parts{i}.name,i-1,parts{i}.name,parts{i}.name,i-1);
        end
        fprintf(fid,'\t];\n');
    end
    
    if C
        fprintf(fid,'\n\t%% compute C\n');
        fprintf(fid,'\tC = zeros([%d,%d]);\n',kDims,3*nJoints);
        for i = 1:nJoints
            fprintf(fid,'\tC(:,%d+(1:3)) = C%s;\n',3*(i-1),joints{i}.name);
        end
    end
    
    if F
        fprintf(fid,'\n\t%% compute F\n');
        fprintf(fid,'\tF = zeros([%d,%d]);\n',kDims,9*nParts);
        for i = 1:nParts
            fprintf(fid,'\tF(%d+(1:7),%d+(1:9)) = F%s;\n',7*(i-1),9*(i-1),parts{i}.name);
        end
    end
    
    if Consts
        for i = 1:nConsts
            printSymMatrix(fid,sprintf('d2constd2t%d',i),consts{i}.d2constd2t,false,0);
            printSymMatrix(fid,sprintf('dconstdq%d',i),consts{i}.dconstdq,false,0);
        end
        fprintf(fid,'\n\t%% compute D\n');
        fprintf(fid,'\tD = zeros([%d,%d]);\n',nConsts,qDims);
        for i = 1:nConsts
            fprintf(fid,'\tD(%d,:) = dconstdq%d;\n',i,i);
        end
        
        fprintf(fid,'\n\t%% compute c\n');
        fprintf(fid,'\tc = zeros([%d,1]);\n',nConsts);
        for i = 1:nConsts
            fprintf(fid,'\tc(%d) = -d2constd2t%d;\n',i,i);
        end
    end
end
