function writeTMTPartInfo(fid,cPart,qDims,sparseThresh,writeForceInfo,writeMassInfo,writeQs,derivOrder)
    if nargin < 5
        writeForceInfo = true;
    end
    if nargin < 6
        writeMassInfo = true;
    end
    if nargin < 7 || writeMassInfo || writeForceInfo
        writeQs = true;
    end
    if nargin < 8
        derivOrder = 2;
    end
    name = cPart.name;
    indentSize = 1;
    inIfDeriv = false;

    fprintf('Writting terms for %s...',name);

    fprintf(fid,'\t%% compute equations for %s\n', name);
    if isfield(cPart,'parentInfo')
        pName = cPart.parentInfo.name;
        printSymMatrix(fid,sprintf('%sParentJoint',name),cPart.parentInfo.parentJointPos,false,0,indentSize);
        printSymMatrix(fid,sprintf('%sJoint',name),cPart.parentInfo.childJointPos,false,0,indentSize);
        printSymMatrix(fid,sprintf('%sRelOrient',name),cPart.parentInfo.relativeOrient,false,0,indentSize);

        if derivOrder > 0
            printSymMatrix(fid,sprintf('d%sParentJointdq',name),cPart.parentInfo.dparentJointPosdq,false,sparseThresh,indentSize);
            printSymMatrix(fid,sprintf('d%sJointdq',name),cPart.parentInfo.dchildJointPosdq,false,sparseThresh,indentSize);
            printSymMatrix(fid,sprintf('d%sRelOrientdq',name),cPart.parentInfo.drelativeOrientdq,false,sparseThresh,indentSize);
        end

        if derivOrder > 1
            printSymMatrix(fid,sprintf('d2%sParentJointd2q',name),reshape(cPart.parentInfo.d2parentJointPosd2q,3,[]),false,sparseThresh,indentSize);
            printSymMatrix(fid,sprintf('d2%sJointd2q',name),reshape(cPart.parentInfo.d2childJointPosd2q,3,[]),false,sparseThresh,indentSize);
            printSymMatrix(fid,sprintf('d2%sRelOrientd2q',name),reshape(cPart.parentInfo.d2relativeOrientd2q,4,[]),false,sparseThresh,indentSize);
        end
        
        printPartDerivs(fid,derivOrder,name,pName,indentSize)
    else
        printSymMatrix(fid,sprintf('%sOrient',name),cPart.Orient,false,0,indentSize);
        printSymMatrix(fid,sprintf('%sCoM',name),cPart.CoM,false,0,indentSize);

        if derivOrder > 0
            printSymMatrix(fid,sprintf('d%sOrientdq',name),cPart.dOrientdq,false,sparseThresh,indentSize);
            printSymMatrix(fid,sprintf('d%sCoMdq',name),cPart.dCoMdq,false,sparseThresh,indentSize);
        end

        if derivOrder > 1
            printSymMatrix(fid,sprintf('d2%sOrientd2q',name),reshape(cPart.d2Orientd2q,4,[]),false,sparseThresh,indentSize);
            printSymMatrix(fid,sprintf('d2%sCoMd2q',name),reshape(cPart.d2CoMd2q,3,[]),false,sparseThresh,indentSize);
        end
    end
    fprintf(fid,'\tR%s = quatrot(%sOrient);\n', name, name);
    if writeQs
        fprintf(fid,'\tQ%s = quatpremat(%sOrient);\n', name, name);
        fprintf(fid,'\tQC%s = quatpostmat(%sOrient);\n', name, name);
        if derivOrder > 0
            fprintf(fid,'\tdQ%sdt = quatpremat(d%sOrientdq*qd);\n', name, name);
        end
    end
    if writeMassInfo
        printSymMatrix(fid,sprintf('j%s',name),cPart.inertia,false,0,1);
        fprintf(fid,'\tJ%s = [zeros(4,1) [zeros(1,3);j%s]];\n', name, name);
        fprintf(fid,'\tI%s = 4*Q%s*J%s*Q%s.'';\n', name, name, name, name);
        fprintf(fid,'\tM%s = diag((%s)*ones([3,1]));\n',name,char(cPart.mass));
    end

    if writeForceInfo
        fprintf(fid,'\t\td2R%sdtdq = reshape(reshape(d2Rd2q*d%sOrientdq*qd,[9,4])*d%sOrientdq,[9,%d]) + reshape(reshape(quatrotderiv(%sOrient)*d2%sOrientd2q,[9*%d,%d])*qd,[9,%d]);\n', ...
            name, name, name, qDims, name, name, qDims, qDims, qDims);
        fprintf(fid,'\t\td2R%sdtdqd = reshape(quatrotderiv(%sOrient)*d%sOrientdq,[9,%d]);\n', ...
            name, name, name, qDims);
        fprintf(fid,'\t\td2CoM%sdtdq = reshape(reshape(d2%sCoMd2q,[3*%d,%d])*qd,[3,%d]);\n', name, name, qDims, qDims, qDims);
        fprintf(fid,'\t\td2CoM%sdtdqd = d%sCoMdq;\n', name, name);

        fprintf(fid,'\n\t%% compute net generalized quaternion space forces for %s\n',name);
        fprintf(fid,'\tfooTerm = 8*(dQ%sdt*J%s*dQ%sdt.'');\n',name,name,name);
        fprintf(fid,'\tfOrient%s = fooTerm*%sOrient;\n', name, name);

        fprintf(fid,'\tF%s = zeros([7,9]);\n',name);
        fprintf(fid,'\tF%s(1:3,1:3) = eye(3);\n',name); % WForce
        fprintf(fid,'\tF%s(4:7,4:6) = 2*Q%s(:,2:4);\n',name,name); % WBTorque
        fprintf(fid,'\tF%s(4:7,7:9) = 2*QC%s(:,2:4);\n',name,name); % WTorque
    end
    fprintf(fid,'\n\n');
    fprintf('done.\n');
end

function printPartDerivs(fid,derivOrder,name,pName,indentSize)
    if nargin < 5
        indentSize = 2;
    end
    fprintf(fid,repmat('\t',[1,indentSize]));
    if derivOrder > 1
        fprintf(fid,['[%sOrient, %sCoM, d%sOrientdq, d%sCoMdq, d2%sOrientd2q, d2%sCoMd2q] = ...\n' ...
            '\t\t\tcomputeHeirarchicalPartInfo(%sOrient, %sCoM, ... \n' ...
            '\t\t\t\t%sParentJoint, %sJoint, %sRelOrient, ... \n' ...
            '\t\t\t\td%sOrientdq, d%sCoMdq, ... \n' ...
            '\t\t\t\td%sParentJointdq, d%sJointdq, d%sRelOrientdq, ... \n' ...
            '\t\t\t\td2%sOrientd2q, d2%sCoMd2q, ... \n' ...
            '\t\t\t\td2%sParentJointd2q, d2%sJointd2q, d2%sRelOrientd2q);\n'], ...
            name, name, name, name, name, name, ...
            pName, pName, name, name, name, ...
            pName, pName, name, name, name, ...
            pName, pName, name, name, name);
    elseif derivOrder > 0
        fprintf(fid,['[%sOrient, %sCoM, d%sOrientdq, d%sCoMdq] = ...\n' ...
            '\t\t\tcomputeHeirarchicalPartInfo(%sOrient, %sCoM, ... \n' ...
            '\t\t\t\t%sParentJoint, %sJoint, %sRelOrient, ... \n' ...
            '\t\t\t\td%sOrientdq, d%sCoMdq, ... \n' ...
            '\t\t\t\td%sParentJointdq, d%sJointdq, d%sRelOrientdq);\n'], ...
            name, name, name, name, ...
            pName, pName, name, name, name, ...
            pName, pName, name, name, name);
    else
        fprintf(fid,['[%sOrient, %sCoM] = ...\n' ...
            '\t\t\tcomputeHeirarchicalPartInfo(%sOrient, %sCoM, ... \n' ...
            '\t\t\t\t%sParentJoint, %sJoint, %sRelOrient);\n'], ...
            name, name, pName, pName, name, name, name);
    end
end

