%%
mass = 1;

initPosition = [1;0];
initVel = [0;2];

nSteps = 30*5;
dt = 1/30;

%% Simulate with explicit constraints
t = 0;
y = [initPosition;initVel];

ys = [];
ts = [];

for i = 1:nSteps
    y = y + dt*constrainedPointMassODE(t,y,mass,[0;0]); % Euler step
    t = t + dt;

    ts = [ts t];
    ys = [ys y];
end

%% Simulate with generalized coordinates
t = 0;
theta = atan2(initPosition(2),initPosition(1));
thetadot = initVel.'*[-sin(theta);cos(theta)];
thetay = [theta;thetadot];

ys = [];
ts = [];

for i = 1:nSteps
    thetay = thetay + dt*tmt_constrainedPointMassODE(t,y,mass,[0;0]); % Euler step
    t = t + dt;

    ts = [ts t];
    ys = [ys [cos(thetay(1));sin(thetay(1))] ];
end

%% Plot
theta = linspace(0,2*pi,100);
figure(1);
% plot(ys(1,:),ys(2,:),'b-',ys(1,1:nSteps/30:end),ys(2,1:nSteps/30:end),'g*',cos(theta),sin(theta),'k--');
plot(ys(1,:),ys(2,:),'b-',ys(1,1:nSteps/30:end),ys(2,1:nSteps/30:end),'g*');
axis equal
axis([-1.5 1.5 -1.5 1.5]);
lineobj = findobj('type', 'line');
set(lineobj, 'linewidth', 4);

% print -depsc constrainedMass-TimePlot.eps

%% Generate animation
for i = 1:nSteps
    figure(1);
    clf;
    plot(ys(1,1:i),ys(2,1:i),'b-',ys(1,1:nSteps/30:i),ys(2,1:nSteps/30:i),'g*',cos(theta),sin(theta),'k--');
    axis equal
    axis([-1.5 1.5 -1.5 1.5]);

    lineobj = findobj('type', 'line');
    set(lineobj, 'linewidth', 4);

%     textobj = findobj('type', 'text');
%     set(textobj, 'fontunits', 'points');
%     set(textobj, 'fontsize', 20);
    drawnow
    
    print('-dpng','-r80',sprintf('constrainedMass-anim%05d.png',i));
end
