function [ret1,ret2] = quataxisangle(in1,in2)
% q = quataxisangle(axis,theta)
%       Create the corresponding quaternion for a rotation of theta about
%       axis.
%
% [theta,axis] = quataxisangle(q)
%       Decompose a quaternion into a rotation of theta about axis.

    if nargin == 2
        axis = in1;
        theta = in2;
        q = zeros([4,1]);
        q(1) = cos(theta/2);
        q(2:4) = (sin(theta/2)/norm(axis))*axis(:);
        ret1 = q;
    elseif nargin == 1
        q = reshape(in1,[4,1]);
        q = q/norm(q);
        ctheta2 = q(1);
        stheta2 = norm(q(2:4));
        axis = q(2:4)/stheta2;
        theta = 2*atan2(stheta2,ctheta2);
        theta(theta > pi) = theta(theta > pi) - 2*pi;
        ret1 = theta;
        ret2 = axis;
    else
        error('Unrecognized number of inputs');
    end
end
