function q = quatmult(q1,q2,diagMult)
% Set q = q1*q2
    if isempty(q1) || isempty(q2)
        q = zeros(4,0);
        return;
    end
    if nargin < 3
        diagMult = false;
    end

    if size(q1,1) ~= 4
        q1 = reshape(q1,4,[]);
    end
    if size(q2,1) ~= 4
        q2 = reshape(q2,4,[]);
    end

    if isnumeric(q1) && isnumeric(q2) && size(q1,2) > 1 && size(q2,2) > 1
        if diagMult
            q = zeros([4,size(q1,2)]);
        else
            q = zeros([4,size(q1,2),size(q2,2)]);
        end
    else
        if size(q1,2) > 1
            q = q1;
        else
            q = q2;
        end
    end
    nqs = numel(q)/4;
    
    if diagMult
        % q =    (      a1a2    -       b1b2    -       c1c2    -       d1d2    )
        q(1,:) = reshape((q1(1,:).*q2(1,:) - q1(2,:).*q2(2,:) - q1(3,:).*q2(3,:) - q1(4,:).*q2(4,:)),[1,nqs]);

        % +      (      a1b2    +       b1a2    +       c1d2    -       d1c2   )i
        q(2,:) = reshape((q1(1,:).*q2(2,:) + q1(2,:).*q2(1,:) + q1(3,:).*q2(4,:) - q1(4,:).*q2(3,:)),[1,nqs]);

        % +      (      a1c2    -       b1d2    +       c1a2    +       d1b2   )j
        q(3,:) = reshape((q1(1,:).*q2(3,:) - q1(2,:).*q2(4,:) + q1(3,:).*q2(1,:) + q1(4,:).*q2(2,:)),[1,nqs]);

        % +      (      a1d2    +       b1c2    -       c1b2    +       d1a2   )k.
        q(4,:) = reshape((q1(1,:).*q2(4,:) + q1(2,:).*q2(3,:) - q1(3,:).*q2(2,:) + q1(4,:).*q2(1,:)),[1,nqs]);
    else
        % q =    (      a1a2    -       b1b2    -       c1c2    -       d1d2    )
        q(1,:) = reshape((q1(1,:).'*q2(1,:) - q1(2,:).'*q2(2,:) - q1(3,:).'*q2(3,:) - q1(4,:).'*q2(4,:)),[1,nqs]);

        % +      (      a1b2    +       b1a2    +       c1d2    -       d1c2   )i
        q(2,:) = reshape((q1(1,:).'*q2(2,:) + q1(2,:).'*q2(1,:) + q1(3,:).'*q2(4,:) - q1(4,:).'*q2(3,:)),[1,nqs]);

        % +      (      a1c2    -       b1d2    +       c1a2    +       d1b2   )j
        q(3,:) = reshape((q1(1,:).'*q2(3,:) - q1(2,:).'*q2(4,:) + q1(3,:).'*q2(1,:) + q1(4,:).'*q2(2,:)),[1,nqs]);

        % +      (      a1d2    +       b1c2    -       c1b2    +       d1a2   )k.
        q(4,:) = reshape((q1(1,:).'*q2(4,:) + q1(2,:).'*q2(3,:) - q1(3,:).'*q2(2,:) + q1(4,:).'*q2(1,:)),[1,nqs]);
    end
end
